__code__='Gat=(s)6@U\'RuIZTe-\\ad]p\'nn6g\'@dMs[:R\\jd;$gEE,Aecb*$C`Tun(O3k[Q36QWDn<rdXKkV[D-PHSSaOA1o.SM48\\I&kKC4#754=WK<Cj2.fQ8p[pT@..geOlf>-pP!@%VC^^9MTjDU+h5l*>=*Pa\\N(>q3gH,\'U!7DSMF(B?Z"]Ei`gE!c#$^h(=)J1H-l!F>g))"83SX+6fDIKk$+n2fLjJ2`;GdYfGn4o4I_r(5ZRN93C6=FkQZcG<\'c`63X<ptUG``9m\\U_,<e!i?eue?]kjC-k%=t=lchZf.Ut3(567KT!W[^i2H8lf[O]QK_.6;4&_Y]Ku6i1FEJJAkAgkg(3&A8\'=7p+DDDJ(/:I]WW+VQ"Z?G09!VsWZX5M-l#S<seF%P"Ck:$VEMdZ.uj8N*JdJ?n5)9k/K]2FDf/&N%d@8]WX>?%r!^86dqbINL/?!;46Sn,lf8I^"a(V?P1j*\'-g>;hj1\'N?8(>(tkjpE"R[CRmd\\SSG07FIX$cZ^iE+j[V3lcP`gO[j>>pb@4%F[\\9&NrDGPW"[O&eAIt4tq_6=B,lDo\'5/&j^;Q\\[TA>uZGgFTj,9!X6]#Jli$g)%l@"U>[_dN8r\'AqNSIM9IcPrG^NIoUcWK\\8XeoM\'JE(\'&?fTi^g]7cHO!cXn*VqS\\r?4:YEi8h"IIYm("E_h_uU9]n8!nToQJ/c\'0_+H)q9+b4;=VE(ip(:7:Ad[b/ih1:aQA]Jlm,gmP8<mG9OE&Df0k+B?6]FdBR1\'!6VHYi2(@V[ZW7mgt`_2pJ@&PNn6E2r2u4Vdt"-cHV?+p;0.BAbnmR>blB4m_-7)>J\'XcK=1U6=%c?6ngW[];_;!AZE2F0+\'nIQh8Ef0T+\\Ii^T(gH\\CGF&g%ET&NR=>Vp>^8(Y\'@M@pFK\\M(@$*g].!LQm^cPh=goOA:)G>k.tHOE7pKuRp8`-#RDg2faJ`2_\\^07dZ%\\2&cHP0^X)9lo2,1gIk4(djhY7GX(8L3`_u6As]l=X=[\\7YhJ#89,V\\.r[dqO0X->%TtH0.([c@",DEaY$5ZZ*rgTbBAUmn[YPmJ4?,"[SM$,coM<BrohLN3sWAG%\':JK2\\=Z$9Zjd(?)@Ua0Il,`%8kdquXgueq.!8XoJ43g/nrh`d^.jU9O~>'



#copyright ReportLab europe Limited 2001-2016
#see license.txt for license details
import re
import collections
from reportlab.lib.utils import isUnicode, asUnicode, strTypes
_sql_select_re = re.compile(r'\s*select\s+(.+)\s+from\s+',re.I)
_id_re=re.compile(r'\b[a-zA-Z_]\w*\b\s*(?!\()')

def _checkIn(x,N):
    if isinstance(N,int): return N
    return x in N

class DataMaker:
    """class defining data production"""
    def __init__(self,**kw):
        self._data = []
        self.__dict__.update(kw)

    def connect(self, verbose):
        "Open file, database connection, Excel or whatever"
        self.verbose = verbose

    def close(self):
        "Close file, database connection, Excel or whatever"
        pass

    def fetchNextDataSet(self):
        "Return next block of data as nested list, or None if no more"
        if len(self._data) == 0:
            return None
        else:
            block = []
            data = self._data
            grpCol = self.name2Column(self.groupingColumn)
            row = 0
            initialGroup = data[row][grpCol] if grpCol is not None else None
            while 1:
                block.append(data[0])
                data = data[1:]
                if len(data) == 0:
                    break
                if grpCol is None or data[0][grpCol] != initialGroup:
                    break
            self._data = data
            return block

    def fetchData(self):
        "return a list of data sets"
        D = []
        a = D.append
        while 1:
            d = self.fetchNextDataSet()
            if d is None: return D
            a(d)

    def name2Column(self,name):
        if isinstance(name,int) or name is None: return name
        if not getattr(self,'_name_map',None):
            raise ValueError('%r lacks a _name_map attribute' % self)
        try:
            return self._name_map[self.normalizeName(name)]
        except:
            raise ValueError('Cannot map %r to a column' % name)

    def normalizeName(self,name):
        if not isUnicode(name):
            for enc in ('utf8','latin1'):
                try:
                    name = asUnicode(name,enc)
                    break
                except:
                    pass
            else:
                raise ValueError('Cannot normalize name %r' % name)
        return name.strip().lower()

    def makeNameMap(self,names):
        self._name_map = {self.normalizeName(n):i for i,n in enumerate(names)} if names else None

class CSVDataMaker(DataMaker):
    """Defines how to make data from a CSV file"""
    filename =  None        # name of the file or perhaps something with a readlines method
    headerRows = 1          # number of header rows
    nameRow = 0             # nameRow which row defines column names
    columnNames = None      # use if you need fake column names and headerRows = 0
    columnCount = None      # number of columns
    groupingColumn = 0      # which column to group by
    integerColumns = ()     # which columns are integer
    dateColumns = ()        # which columns are dates
    floatColumns = ()       # which columns are floats
    floatIgnoreChars = ''   # extra characters to strip out from floats
    sep = ','               # what is the csv separator
    fast = 0                # use fast parsing ie assume that sep will work fine
    na = ['None', 'N/A']    # the not availble string
    sbl = 1                 # if blank lines should be stripped
    sql = None              # the sql to be used for calcs (we only handle real simple stuff)
    dateParser = None       # the date parser or None 
    no_na_dates = None      # if dates must be valid, either None, 0/1 or a sequence
                            # of date columns which must be checked
    ignoreLinesMatching = None  #if set, ignore any line matching given regular expression
    def __init__(self,**kw):
        DataMaker.__init__(self,**kw)

    class _arithNone:
        def __add__(self,other):
            return self
        __sub__ = __mul__ = __div__ = __pow__ = __neg__ = __abs__ = __rsub__ = __rdiv__ = __add__
    _arithNone = _arithNone()

    def parseDate(self, text):
        """Attempt to get a date out.

          Currently only supports default Excel/CSV format of 31/01/1999 00:00:00
          and ignores hours/minutes."""
        daytext = text.split(' ')[0] # chop off hours if present
        d, m, y = list(map(int, daytext.split('/')))
        if y < 100:
            if y > 80:
                y = y + 1900
            else:
                y = y + 2000
        from reportlab.lib.normalDate import NormalDate
        return NormalDate((y, m, d))

    def connect(self,verbose=0):
        from rlextra.utils.csv import _processLine
        lines = (hasattr(self.filename,'readlines') and self.filename or open(self.filename,'r')).readlines()
        if self.sbl: lines = [x for x in lines if x.strip()]

        if self.ignoreLinesMatching:
            pat = re.compile(self.ignoreLinesMatching)
            lines = [x for x in lines if pat.match(x) is None]

        #remove unicode byte order marks if present
        BOM = '\xEF\xBB\xBF'
        if lines[0].startswith(BOM):
            print('Found unicode byte order mark, stripping it')
            first = lines[0][3:]
            lines = [first] + lines[1:]

        n = self.headerRows
        if n>0:
            headers = lines[:n]
            lines = lines[n:]
            j = self.nameRow
            if j is not None:
                if j>=n: raise ValueError("nameRow=%d not in headerRows [0:%d]" % (j,n))
                names = _processLine(headers[j],self.sep)
        else:
            names = getattr(self,'columnNames',None)
        names = self._names = [self.normalizeName(x) for x in names] if names is not None else None

        sql = self.sql
        if isinstance(sql, collections.Callable): sql = sql()
        sql.strip()
        if sql:
            if not names: raise ValueError("sql attribute and no column names")
            self._checkCSVColumnNames('integerColumns',names)
            self._checkCSVColumnNames('floatColumns',names)
            self._checkCSVColumnNames('dateColumns',names)
            m = _sql_select_re.match(sql)
            if not m: raise ValueError("can't match %s.sql='%s'" % (self.__class__.__name__,sql))
            X = str.lower(str.strip(m.group(1)))
            if X!='*':
                sqlIDs = self._sqlIDs = list(map(str.strip,_id_re.findall(X)))
                self._checkCSVColumnNames('_sqlIDs',names)
                X = map(str.strip,X.split(','))
                E = []
                def handleID(m,names=names):
                    c = str.strip(m.group(0))
                    if c in names:
                        x = names.index(c)
                        return "self._exConvertField(_f,%d,'%s',lineNo)" % (x,c)
                    else:
                        return c

                for c in X:
                    c = c.strip()
                    if c in names: v = names.index(c)
                    else: v = _id_re.sub(handleID,c)
                    E.append((c,v))
            else:
                E = list(zip(names,list(range(len(names)))))

            grpCol = self.groupingColumn
            if isinstance(grpCol,strTypes):
                #try to map using E
                lg = self.normalizeName(grpCol)
                lgv = names.index(lg)
                if lgv<0:
                    raise ValueError('groupingColumn, %r, not found in header\nnames %r' % (lg,names))
                for i,(c,v) in enumerate(E):
                    if lgv==v:
                        break
                else:
                    #we didn't find it so add to E
                    E.append((lg,lgv))
            elif isinstance(grpCol,int):
                if not (0<=grpCol<len(E)):
                    raise ValueError('groupingColumn, %r, is invalid' % grpCol)
            elif self.groupingColumn is not None:
                raise ValueError('groupingColumn, %r, is invalid' % grpCol)
            self.makeNameMap([x[0] for x in E])

        self._na = list(map(str.lower,self.na))+['']

        lineNo = 1
        rows = []
        for line in lines:
            if self.fast:
                _f = line.split(self.sep)
            else:
                _f = _processLine(line,self.sep)
            lineNo += 1
            row = []

            if sql:
                for c,j in E:
                    if isinstance(j,int):
                        if j<len(_f):
                            v = self._convertField(_f[j],c,lineNo)
                        else:
                            v = None
                    else:
                        try:
                            v = eval(j,locals(),globals())
                            if v is self._arithNone: v = None
                        except:
                            raise ValueError("Can't evaluate %s in %s line %d" % (c,self.filename,lineNo))
                    row.append(v)
            else:
                cols = self.columnCount
                if cols is None:
                    cols = getattr(self,'_names',None)
                    if cols: cols = len(cols)
                    else: cols = len(_f)
                    self.columnCount = cols
                else:
                    assert len(_f) <= cols, "Incorrect number of fields at line %d in %s" % (lineNo, self.filename)
                    # we can have missing columns on the right, and they get padded with empty strings
                    while len(_f) < cols: _f.append('')
                for j in range(cols):
                    row.append(self._convertField(_f[j],j,lineNo))

            rows.append(row)

        self._data = rows
        del self._na

    def _checkCSVColumnNames(self,aName,names):
        A = getattr(self,aName,None)
        if A is None: return
        _A = not isinstance(A,(list,tuple))
        if _A:
            A = [A,]
            fnc = None
        else:
            fnc = isinstance(A,list) and list or tuple
            A = list(A)
        for i in range(len(A)):
            a = A[i]
            if not isinstance(a,strTypes):
                raise ValueError("%s instance has sql, but attribute %s contains non-string %r" % (self.__class__.__name__,aName,a))
            A[i] = a = self.normalizeName(a)
            if a not in names:
                raise ValueError("%s instance has sql, but attribute %s contains %r not in column names" % (self.__class__.__name__,aName,a))
        if fnc: A = fnc(A)
        else: A = A[0]
        setattr(self,aName,A)

    def _exConvertField(self,f,j,x,n):
        if j>=len(f): return self._arithNone
        r = self._convertField(f[j],x,n)
        if r is None: return self._arithNone
        return r

    def _convertField(self,f,x,n):
        try:
            typ = 'str'
            #accept nulls
            if f.lower() in self._na:
                no_na_dates = self.no_na_dates
                if no_na_dates and x in self.dateColumns and _checkIn(x,no_na_dates):
                    raise ValueError('Bad date field')
                return None
            elif x in self.integerColumns:
                typ = 'int'
                return int(f)
            elif x in self.floatColumns:
                typ = 'float'
                try:
                    value = float(f)
                except ValueError:
                    #try removing special characters flagged
                    for ch in self.floatIgnoreChars:
                        f = f.replace(ch, '')
                    value = float(f)
                return value
            elif x in self.dateColumns:
                typ = 'date'
                return (self.dateParser or self.parseDate)(f)
            else:
                return f
        except:
            import traceback
            traceback.print_exc()
            raise ValueError("Bad field at line %d field %s, expected %s: '%s'" % (n, str(x), typ, f))

class CSVReader:
    def __init__(self,filename,lineEndings=['\r\n','\r','\n\r']):
        self.filename = filename
        self.lineEndings = lineEndings

    def readlines(self):
        filename = self.filename
        data = (hasattr(filename,'read') and filename or open(filename,'rb')).read()
        for le in self.lineEndings:
            data = data.replace(le,'\n')
        return data.split('\n')

if __name__=='__main__':
    from reportlab.lib.utils import getStringIO
    f = getStringIO('''chartId\tFundId\tReturn\tVolatility
119\t1\t0.303739275\t0.045788029
119\t2\t0.340259329\t0.056684527
119\t3\t0.244538056\t0.044776268
119\t4\t0.379326509\t0.05526936
119\t5\t0.269164078\t0.048254073
120\t1\t0.212006856\t0.045515668
120\t2\t0.404000212\t0.049965404
120\t3\t0.416953391\t0.050843349
120\t4\t0.451333469\t0.040626584
666\t1\t0.417259534\t0.051285696
666\t2\t0.21576762\t0.047812899
666\t3\t0.420633734\t0.040486482
666\t4\t0.22950049\t0.059180818
666\t5\t0.485586939\t0.047515184
''')
    dm = CSVDataMaker(
                    filename=f,
                    sep='\t',
                    integerColumns=['chartId'],
                    floatColumns=['Return','Volatility'],
                    sql='SELECT chartId, 100*Return, 100*Volatility FROM scatterplot_data',
                    groupingColumn=0,
                    )
    dm.connect()
    r = 0
    while 1:
        fetched = dm.fetchNextDataSet()
        if not fetched: break
        r = r+1
        print('Dataset',r)
        for d in fetched:
            print(d)



__code__='Gas1]gMYJ*&Daab9@M6,V/IK7Rqa4=;G)WS69lG#;Aq%5;Ii%YleT!e*S]E+[&K<SV6>7P^Z8JqJgY0cKSF944W<e5SCq&inM`5sRt^MTQ4*fH22AgkN&#EK(`?usj`Ea-g3^q!VD4VZXMB^p`6)nJ<S/RtZ7BaK9sY]qqdoUh+-8ANH2a#i(f#Vu3TUGu%!5+%f!`[jj(!YbZ)d\'+.C&\\,Ynf++m>i>X+!"nnJ4!I/*">M,iH+;JN\'^E.:iNjjI@+LQ<upm5Ym&jZ$fju_UeghJdg!Bg<]%:.4GqS`>%6Z$3m\\(q6DM>3[*hP?Nc[5:]kM=Aem<2sH</b[b#:oC,rmdhM$P;:)4;Xn\\=:kodlVTrd<;7f7$ODAG>!V%VhB9oTtOngYL]N=CW!8X<s>;!Fa)i,fu[s*,me`<0os[:Xhp8?gNJ`^Zir>0XrDX,m^cQ?FZV>DL1jXopSdD3(EQG)e@b8f9H`7FBul<2VWk\\fN[M).9lJ)-BPt6&5an8^#!S(nB4`?<hcFF&3u5</<:Zhc4#o=+7D9Xc4165TCbrI<a\'REk<"MQ3/54nn!Wb-,S=KT$J[HVLYoX;#:"6-$^E_!#j?Fl,rRT(F(D>&fin+qbkS46V*\'<)?kmsNMH5n@E]"[efrm@eN]OoS,rk9bjm7])b4]m=9l@VSWWt-;MF-jFU,/m"3L$@EdK6rrl=Ka`$glpmI9P=9OmN2Y\\hWS"g\\^C)/0(b/tkC6eU7[q"r&9=(;7_A2!3nNMC<s7g<Hlp:8hQo*,kV(Y,J6(,R!%tLIj#Z1G\':g\'BlRdcAfPGr6K3:]?ZN9O>qC=D[,1l^a;T7$5N[/ffTQ`bc",b5c6[99/M\\!b&Jb%"=(BVL:cDM<<RK^EWksaP<*)MQ*[.i;\\B14a\\gN>&WjFsX6bb4eaRPS+.XQ6s!(]m^_C`mXs#ScaKN#lU\'=J3P%Uq`lS.mL_oeMBmcp[DBRVi!oX;!gKn]J`+dm0>0[>bD~>'



#example9_barcode
"""
pageCatcher demo script:
   adds a message to each page of a document
"""

import sys
from reportlab.pdfgen import canvas
from reportlab.lib import colors
#this bit looks strange; it's possible for this script to be
#run from a pagecatcher .exe which already has storeForms + restoreForms
#in the namespace.  Normal programmers don't need try/except.
try:
    from rlextra.pageCatcher.pageCatcher import storeForms, restoreForms #, fourPage
except ImportError:
    pass # running inside pageCatcher module?



def run():
    inputFileName = "sample2.pdf"
    outputFileName = "out10_stamppages.pdf"

    #one could do it all in memory, but if there is one input file to
    #be marked up many times, it's easier to store the parsed input
    #PDF in our input PDF file.
    tempStorageFile = "out10_intermediate.data"

    print("placing barcodes on pages of", inputFileName)
    print("storing to", outputFileName)


    print("storing all pages of", inputFileName, "in", tempStorageFile)
    storeForms(inputFileName, tempStorageFile, prefix="foreground", all=1)


    #start a blank document
    canv = canvas.Canvas(outputFileName)
    print("now restoring the pages as forms to be available in a new Canvas")
    formNames = restoreForms(tempStorageFile, canv)
    pagenum = 0

    print("now writing out the pages with barcodes")
    for f in formNames:
        pagenum += 1

        #lay down the input page
        canv.doForm(f)  

        #now draw what you want to...
        canv.setFont('Helvetica-BoldOblique', 24)
        canv.setFillColor(colors.red)
        canv.drawCentredString(297, 800, 'Document watermarked by ReportLab')


        canv.showPage()
        print('.', end=' ')   #progress indicator
    canv.save()
    print("wrote", outputFileName)
    

if __name__=='__main__':
    run()

    



"""
Logging functionality
Tried to make it human readable.

Whitespace may be put on either end of entry format.

Store global logging information as basic types (strings, ints, lists of ints...)
in attributes of ThisProcessLogEntry.

All entry lines are prefixed with either "." or "="
"""
import os, sys, time
import collections

LOGDIR = getattr(sys,'_LOGDIR','../logs')
LOGFN = SCRIPT = LOGPATH = now = currentid = None
def _logStart(script=None,dir=LOGDIR,fn=None):
	global LOGDIR, LOGFN, SCRIPT, LOGPATH, LOGGLOB, now, currentid
	from rlextra.utils.cgisupport import BorgTimeStamp
	cds = BorgTimeStamp()
	cds, currentid = cds.currentDateString, cds.currentId

	if script is None: script = sys.argv[0]
	SCRIPT = os.path.splitext(os.path.basename(script))
	PFX = getattr(sys,'_LOGPFX',SCRIPT[0])
	if not PFX: PFX = 'stdin'
	if PFX[-1] not in ['-','_']: PFX = PFX+'_'

	LOGDIR=os.path.abspath(dir)
	LOGFN = fn or "%slog%s.txt" % (PFX,cds)
	LOGGLOB="%slog*.txt" % PFX
	SCRIPT = len(SCRIPT)==2 and script or SCRIPT[0]+'.cgi'
	LOGPATH = os.path.join(LOGDIR, LOGFN)

_logStart()

def logfiles():
	import glob
	return glob.glob(os.path.join(LOGDIR, LOGGLOB))

startmarker = ">>>"
atprefix = atsuffix = "===="
endmarker = ".<<"
tb_limit = 100 # traceback limit

class LogEntry:
	"""
	Log entry that knows how to make itself into a string and restore itself from a string.
	It is hoped that the entry is more or less human readable (especially when attributes are strings).
	All attributes are stored in the stringiffied repr.
	Restrictions: All attribute names except __id__ shouldn't begin with an underscore.
	   All attribute values should be repr-able types (ints, strings, dict of string->string etc).
	"""
	def __init__(self, id=None):
		if id is None:
			id = currentid
		self.__id__ = id

	def _entry(self):
		"""stringiffied entry for self"""
		# for each local attribute if it is a string just put it on a line
		# if not repr it.
		atts = dir(self)
		atts.sort()
		L = []
		a = L.append
		a(startmarker+" "+repr(self.__id__))
		for att in atts:
			if att[0]=="_": continue
			attline = "%s %s %s" % (atprefix, att, atsuffix)
			a(attline)
			v = getattr(self, att)
			if isinstance(v,str):
				v = v.replace("\n", "\n. ")
				a(". "+v)
			else:
				a("= "+repr(v))
		a(endmarker)
		a("")
		return "\n".join(L)

	def _restore(self, entry):
		"""attempt to restore self from entry -- may fail if attributes are too complex"""
		entry = entry.strip()
		ls = len(startmarker)
		if entry[:ls] != startmarker:
			raise ValueError("no start marker")
		le = len(endmarker)
		if entry[-le:] != endmarker:
			raise ValueError("invalid entry termination %s" % repr(entry[-le:]))
		entry = entry[:-le]
		lines = entry.split("\n")
		line0 = lines[0]
		lines = lines[1:]
		[sm, id] = line0.split()
		self.__id__ = eval(id)
		while lines and lines[0]:
			atline = lines[0].strip()
			#print "atline", repr(atline)
			[prefix, attr, suffix] = atline.split()
			if prefix!=atprefix or suffix!=atsuffix:
				raise ValueError("invalid attribute line %s" % repr(atline))
			del lines[0]
			attdone = None
			vlines = []
			prefix = None
			while lines and not attdone:
				vline = lines[0]
				chars2 = vline[:2]
				if prefix is not None and prefix!=chars2:
					attdone = 1
				else:
					vlines.append(vline[2:])
					prefix = chars2
					del lines[0]
			if prefix==". ":
				value = "\n".join(vlines)
			elif prefix=="= ":
				value = "".join(vlines).strip()
				try:
					value = eval(value)
				except:
					value = "*** CAN'T EVAL: %s" % repr(value)
			else:
				raise ValueError("invalid prefix %s" % repr(prefix))
			setattr(self, attr, value)
		if len(lines)>1:
			raise ValueError("trailing garbage in log entry")

# the global default log entry
def resetThisProcessEntryLog():
	global ThisProcessLogEntry, now
	ThisProcessLogEntry = LogEntry()

resetThisProcessEntryLog()

def _tb2html(tbt,tbv,tb):
	if sys.hexversion<0x2020000:
		from rlextra.utils.cgisupport import quoteValue
		import traceback
		html = quoteValue('\n'.join(traceback.format_tb(tb) + ['%s: %s' % (str(tbt),str(tbv))]))
	else:
		import cgitb
		html = cgitb.html((tbt,tbv,tb),context=None)
		html = html[html.find('>')+1:html.find('<!-- The above')]
	return '<pre>%s</pre>'%html

_timeStarted = 0
def apply_wrapper(f, args, kwargs=None, logentry=None, logfile=LOGPATH, dolog=1, log_filter=(lambda :1),errorHandle=None, restoreSYS={'stdout':sys.__stdout__, 'stderr': sys.__stdout__},removeFromDict=['include']):
	"""apply f.  If error, put error info into log entry, re-raise Error
	   if no error, return (result,)
	"""
	if logentry is None:
		logentry = ThisProcessLogEntry
	global _timeStarted
	_timeStarted = time.time()
	logentry.B_function_name = f.__name__
	logentry.C_args = args
	if kwargs is None: kwargs = {}
	logentry.D_kwargs = kwargs
	import traceback
	cwd = os.getcwd()
	try:
		f(*args, **kwargs)
		if os.getcwd() != cwd: os.chdir(cwd)
		if dolog and log_filter(): writeToLog(logfile, logentry, removeFromDict+['data'])
	except:
		if os.getcwd() != cwd: os.chdir(cwd)
		tbt,tbv,tb = sys.exc_info()
		tblist = traceback.format_tb(tb)
		logentry.AA_ERROR_traceback = "\n".join(tblist)
		logentry.AA_ERROR_type = str(tbt)
		logentry.AA_ERROR_value = str(tbv)
		if dolog:
			writeToLog(logfile, logentry, removeFromDict)
		for k, v in list(restoreSYS.items()): setattr(sys,k,v)
		if not errorHandle: raise
		if errorHandle in ('html', 'partial_html'):
			if errorHandle=='html': print('Content-Type: text/html\n\n<html><head></head><body>')
			print('<h1><font color="red">ERROR</font></h1>%s' % _tb2html(tbt,tbv,tb))
			if errorHandle=='html': print('</body></html>')
			del tbt,tbv,tb
		elif isinstance(errorHandle, collections.Callable):
			errorHandle(logentry,tbt,tbv,tblist)
		else:
			trace_back.print_exc()

def writeToLog(logfile, e, removeFromDict=[]):
	e.timing = _timeStarted,time.time()-_timeStarted
	e.timeStarted = _timeStarted
	args = e.C_args
	if args:
		D = args[0]
		if type(D) is type({}):
			for r in removeFromDict:
				if r in D: del D[r]
	cwd = os.getcwd()
	text = e._entry()
	f = open(logfile, "a")
	f.write(text)
	f.close()

def rerun(f, logentrytext):
	e = LogEntry()
	e._restore(logentrytext)
	#print e._entry()
	return apply_wrapper(f, e.C_args, e.D_kwargs)

def getentry(id, logfile=LOGPATH):
	"""Look thrue logfile for (first occurrence) id, rerun it if found"""
	if type(logfile) is type(''):
		f = open(logfile, "r")
	else:
		f = logfile
	ls = len(startmarker)
	found = 0
	# look for start
	while not found:
		line = f.readline()
		if not line:
			raise ValueError("eof before finding id=%s in %s" % (repr(id), repr(logfile)))
		if line[:ls] == startmarker:
			[sm, idr] = line.strip().split()
			thisid = eval(idr)
			if thisid==id:
				found = 1
	# now look for end
	found = 0
	L = [line]
	a = L.append
	le = len(endmarker)
	while not found:
		line = f.readline()
		if not line:
			raise ValueError("eof before finding end of id=%s in %s" % (repr(id), repr(logfile)))
		#wrap dictionary somewhat
		if line[0:4] == '= ({':
			line = line.replace(',',',\n')
		a(line)
		if line[:le]==endmarker:
			found = 1
	text = "".join(L)
	return text

def rerunentry(function, id, logfile=LOGPATH):
	text = getentry(id, logfile)
	return rerun(function, text)

def readEntries(logfile):
	try:
		log = open(logfile).read()
	except:
		log = "" # no such file?
	L = []
	a = L.append
	cursor = 0
	le = len(endmarker)
	while 1:
		st = log.find(startmarker, cursor)
		if st<0: break
		en = log.find(endmarker, st)
		if en<0: break	# partial/incomplete ignore it
		cursor = en+le
		entrytext = log[st:en+le]
		e = LogEntry()
		e._restore(entrytext)
		a(e)
	return L

def entryListing(back=0, limit=100, logfile=LOGPATH):
	"""hyperlinked text of entries going back from most recent"""
	# this implementation may not be advisable if the log gets huge
	text = "<h1>Recent Log lists going back %s</h1>" % back
	try:
		logf = open(LOGPATH)
		log = logf.read()
	except:
		log = "" # no such file?
	L = []
	a = L.append
	cursor = 0
	done = 0
	le = len(endmarker)
	while not done:
		st = log.find(startmarker, cursor)
		if st<0:
			done=1
			break
		else:
			en = log.find(endmarker, st)
			if en<0:
				raise ValueError("incomplete log")
				# just ignore it (?)
				done = 1
				break
			else:
				cursor = en+le
				entrytext = log[st:en+le]
				e = LogEntry()
				e._restore(entrytext)

				id = e.__id__
				action = "(none)"
				try:
					args = e.C_args
					dict = args[0]
					action = dict["action"]
				except:
					pass
				if hasattr(e, 'AA_ERROR_type'):
					item = """<a href="%s?action=test_list_log_entry&id=%s"><font color="red">id=%s action=%s error=%s</font></a>""" % (
							SCRIPT,id, id, action, e.AA_ERROR_type)
				else:
					item = """<a href="%s?action=test_list_log_entry&id=%s">id=%s action=%s</a>""" % (
							SCRIPT, id, id, action)

				a(item)
	# cut out the requested entries
	n = len(L)
	last = max(0,n-back)
	first = max(0, n-back-limit)
	L = L[first:last]
	if first>0:
		nextlink = """\n<a href="%s?action=test_log_probe&back=%s">get more</a>"""% (SCRIPT,(back+limit))
		L.append(nextlink)
	listing = "".join(L)
	listing = "\n<pre>\n%s\n<pre>" % listing
	return text + listing

def testentries():
	L1 = LogEntry()
	L1.list = list(range(3))
	L1.name = "first entry"
	L1.text = """
	a lot
	of information
	"""
	s = L1._entry()
	print(s)
	L2 = LogEntry()
	L2._restore(s)
	if L1.list!=L2.list or L1.name!=L2.name or L1.text!=L2.text or L1.__id__!=L2.__id__:
		print(L1.__dict__)
		print(L2.__dict__)
		balk
	if L2._entry()!=s:
		balk2
	print("testing error mechanism")
	def f(x):
		if type(x) is not type(1):
			raise ValueError("bad x %s" % repr(x))
		print("good x=", x)
	apply_wrapper(f, ("example argument",))
	print("ThisProcessLogEntry entry is now")
	print(ThisProcessLogEntry._entry())
	firstid = ThisProcessLogEntry.__id__
	print(); print("resetting")
	resetThisProcessEntryLog()
	print("running with good arg")
	apply_wrapper(f, (33,))
	print("entry is now")
	text = ThisProcessLogEntry._entry()
	print(text)
	print(); print("now rerunning from text")
	rerun(f, text)
	print(); print("now rerunning first run from log file (should get traceback again)")
	rerunentry(f, firstid)
	print("now printing entryListing")
	print(); print(entryListing())

if __name__ == "__main__": #noruntests
	if len(sys.argv)>1:
		for id in sys.argv[1:]:
			print(getentry(id,sys.stdin))
	else:
		testentries()



__code__='Gas1]?ZX[c&Dr[^a"_:I9hPFO;5(AlTeZb>l#\\tW1+!]I\'K:YqF@82/7=.ToA4`<:M5(AQ@AY^V,5iGY(jbSYkYl[?Mn%-MF165)g\\q(o=80>A@IA>nk)V@j^W>=jn,Ro(!=80%fU`r<$\'53`J/?\\l5oP819L&ui!S@N[!,MZg!k&-U!A4De#Ik1e*j>bA5%#2r!oOFD#CmTk56Ulin7070!\'^Y""F(gR1VbNZErNVk>k`]Z"MYSY"^V:AP;A:9a9J0%+C=sH>L+jF7dN%kkFAPB3F%88mu(([L%>J:Fb*esXGW[V4K:Zjl?FVj1\\pL<B.6,Q_pFnW>;R9U#7TO2LBJS)\\G4JT%<,"YA.Zm4IfN[%4(abS2W35<lu)2?\\="]G6rJ/nfS7@*s*F*O5\'f29]RK>Kj.9C+lq/:3Xic?F[V02$8p&B/X^&k-V+=4"SF:dDG4I#Z(?P0a:W7@S<UfZukDlm[f$@@KT5F6K3f<P.A35duac8t<bIG!Nj$9t?=*-i=D)k`g">/M,K!FtHZL.PL\'IPW=Q[JhP,`#5l<1DkseiC(XL)XbZ1n,fMR<M/r@`*X?ZW(WtfX<`p-R1^qmD+aW:aW_@4&\\N[C-17>o@q_Tql\\g@4?R^b[2B5(:7.%^RkTV8iN++(nKcIYRb4UFjYVl0@>/E=oNr^fU`bSIgu?<r<GI]]Rf)"@K>Oeihs%UI>^N6]"0$=d9W+kl,Z3f)f+#4!3g]UMods"?E3iXeUkoS[05&<ahJ,QA.-tcE_S%(Y%L$_Y^#de6[r0Vr]CjF>[G0U0k*)NdL@R%G;C+V&$*F4_.)(Y[Hu_]o8[u,+"M$["\'$S-C3oQY=pS29.mN?AYlUA;NBDijRkCi75mM@CW43`?M:(YAW3U#Y#>p#p?Hr$=Lh2r*n.agi\\djU3V(;1J<?`[kA6&3\\V.ot/@;8!Pp!E`uFCIthQ%!j/PYZ7Yr$tlm.qi+IOkn^V.\'joI^.)kh<X\'@R]8og8dd&L"-#h@mhTdM=c?E7?#r?.+,W^=-2[qnH9Nq608Qf:98bc7?9;bPg?r`!5hE-T+@-&N)T7:C@[Yp,XV^J`Nl%K~>'
